package packageForLT06;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;

/**
 * <p>
 * The class encapsulates a view of a SimpleThreeDotModel (where a "view" is a
 * visual representation of data). This view needs to be added to a widget
 * container.
 * </p>
 * 
 * <p>
 * The view consists of a bounded two-dimensional plane. The dots in the data
 * model are each drawn on the plane in their specified location and with their
 * specified size. The user space is mapped directly to the device space.
 * </p>
 * 
 * <p>
 * The only constructor this class has is one that requires a
 * SimpleThreeDotModel object. It doesn't make sense to create this view without
 * its data model.
 * </p>
 * 
 * <p>
 * This view is capable of listening to data models and will install itself on
 * the passed data model. Any time the data model changes, this view will
 * repaint itself.
 * </p>
 * 
 * @author mb
 * 
 */
public class DotDrawingWidget extends JPanel implements DotModelListener {
	private DotModel theModel;
	private int width = 400;
	private int height = 400;
	private boolean isFillOn = false;

	private DotDrawingWidget() {
	}

	/**
	 * Constructs a view as specified in the class description. The view gets
	 * installed upon the passed model.
	 * 
	 * @param model
	 */
	public DotDrawingWidget(DotModel model) {
		theModel = model;
		// does this view need to update itself in response to any changes
		// in the model? If so, install a listener here
		model.addListener(this);

	}

	/**
	 * This method renders the graphics of this view.
	 * 
	 * The method is invoked any time the view is repainted. Here is what the
	 * method actually does. We can see that it causes the whole view to be
	 * drawn anew. (Obviously, this is breaking the encapsulation, but is
	 * included for pedagogical reasons). <br />
	 * <code>
	 * 		Graphics2D g2 = (Graphics2D) g;<br />
	 * 		g2.clearRect(0, 0, width, height); // this clears the
	 * 		// drawing space <br />
	 * 		for (int i = 0; i < theModel.getNumberOfDots(); i++) {<br />
	 * 			System.out.println(i);<br />
	 * 			g2.setColor(theModel.getColour(i));<br />
	 * 			g2.draw(theModel.getDot(i));<br />
	 * 		} <br />
	 * </code>
	 */
	public void paintComponent(Graphics g) {
		// super.paintComponent(g); // yes, this should be the first statement,
		// but I want some parallelism with Version 5

		Graphics2D g2 = (Graphics2D) g;
		g2.clearRect(0, 0, width, height); // this clears the
		// drawing space
		for (int i = 0; i < theModel.getNumberOfDots(); i++) {
			g2.setColor(theModel.getColour(i));
			if (isFillOn()) {
				g2.fill(theModel.getDot(i));
			}
			g2.draw(theModel.getDot(i));
		}

	}

	private boolean isFillOn() {
		return isFillOn;
	}

	public Dimension getPreferredSize() {
		return new Dimension(width, height);
	}

	/**
	 * <p>
	 * Method that changes this view in response to any change in the data
	 * model. If the data model changes, the view repaints itself anew.
	 * <p/>
	 * 
	 * <p>
	 * This method is invoked to signal to this listener that the model (upon
	 * which this listener has been installed) has been changed. This provides
	 * an opportunity for the listener to something in response to any change to
	 * the model.
	 * </p>
	 * 
	 * <p>
	 * <b>Who invokes this method?</b> Think of who has been given access to
	 * this view... The documentation for the constructor specifies that this
	 * view has been installed upon the specified data model. So the data model
	 * is able to invoke this method within its inner workings.
	 * </p>
	 */
	public void changed() {
		// what should happen if we come to learn that the model has changed? we
		// should repaint ourself.
		this.repaint();
	}

	public void setFillOn() {
		isFillOn = true;
	}

}
